/* TTMPCli.c
 *
 * General TTMP client.
 *
 * Copyright (C) 2005-2006 TomTom BV <http://www.tomtom.com/>
 * Author 0: Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2
 * as published by the Free Software Foundation.
 */

#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "TTMP.h"
#include "TTMPTestPlat.h"

#define DUMP_MESSAGE

const char *progname = "ttmpcli";

NORETURN static void usage(const char *fmt, ...)
{
	va_list ap;

	fprintf(stderr, "Command line error: ");
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	fprintf(stderr,
		"\n"
		"\n"
		"Usage: %s address [field...]\n"
		"\n"
		"Each field consists of:\n"
		"- A name of at most %d characters\n"
		"- One of these type names:\n"
		"- A value\n"
		"separated by semicolons.\n",
		progname, TTMP_MAX_FIELDNAME);
	exit(1);
}

NORETURN static void die(const char *fmt, ...)
{
	va_list ap;

	fprintf(stderr, "%s fatal error: ", progname);
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	fprintf(stderr, "\n");
	exit(1);
}

int main(int argc, char *argv[])
{
	const char *addr;
	TTMPMessage msg;
	int res;
	TTMPConnection conn;
	int i;
	char *type;
	char *val;
	size_t namelen;
	longlong l;
	float f;
	double d;

	signals_init();
	sockets_init();

	if (argc <= 2)
		usage("not enough arguments");

	addr = argv[1];

	res = TTMPMessageCreate(&msg);
	if (res != TTMP_ERR_NONE)
		die("unable to create message, error %d", res);

	res = TTMPConnectionCreate(&conn, TTMP_TO_DEFAULT, 1, 1);
	if (res != TTMP_ERR_NONE)
		die("unable to create connection, error %d", res);

	for (i = 2; i < argc; ++i) {
		type = strchr(argv[i], ':');
		if (type == NULL)
			usage("no semicolon in field \"%s\"", argv[i]);
		namelen = type - argv[i];
		*type++ = '\0';

		val = strchr(type, ':');
		if (val == NULL)
			usage("only one semicolon in field \"%s\"", argv[i]);
		*val++ = '\0';

		/* Check field name */
		if (namelen == 0)
			usage("name in field \"%s\" too short", argv[i]);
		if (namelen > TTMP_MAX_FIELDNAME)
			usage("name in field \"%s\" too long", argv[i]);

		/* Check field type */
		if        (strcasecmp(type, "int8"  ) == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < INT8_MIN || l > INT8_MAX)
				usage("invalid Int8 field value \"%s\"", val);
			res = TTMPSetFieldInt8(msg, argv[i], (TInt8) l);
		} else if (strcasecmp(type, "uint8" ) == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < 0 || l > UINT8_MAX)
				usage("invalid Uint8 field value \"%s\"", val);
			res = TTMPSetFieldUint8(msg, argv[i], (TUint8) l);
		} else if (strcasecmp(type, "int16" ) == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < INT16_MIN || l > INT16_MAX)
				usage("invalid Int16 field value \"%s\"", val);
			res = TTMPSetFieldInt16(msg, argv[i], (TInt16) l);
		} else if (strcasecmp(type, "uint16") == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < 0 || l > UINT16_MAX)
				usage("invalid Uint16 field value \"%s\"", val);
			res = TTMPSetFieldUint16(msg, argv[i], (TUint16) l);
		} else if (strcasecmp(type, "int32" ) == 0) {
			if (l < INT32_MIN)
				(void) 0;
			if (l > INT32_MAX)
				(void) 0;
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < INT32_MIN || l > INT32_MAX)
				usage("invalid Int32 field value \"%s\"", val);
			res = TTMPSetFieldInt32(msg, argv[i], (TInt32) l);
		} else if (strcasecmp(type, "uint32") == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < 0 || l > UINT32_MAX)
				usage("invalid Uint32 field value \"%s\"", val);
			res = TTMPSetFieldUint32(msg, argv[i], (TUint32) l);
		} else if (strcasecmp(type, "real32") == 0) {
			if (sscanf(val, "%f", &f) != 1)
				usage("invalid Real32 field value \"%s\"", val);
			res = TTMPSetFieldReal32(msg, argv[i], (TReal32) f);
		} else if (strcasecmp(type, "real64") == 0) {
			if (sscanf(val, "%lf", &d) != 1)
				usage("invalid Real64 field value \"%s\"", val);
			res = TTMPSetFieldReal64(msg, argv[i], (TReal64) d);
		} else if (strcasecmp(type, "real"  ) == 0) {
			if (sscanf(val, "%lf", &d) != 1)
				usage("invalid Real field value \"%s\"", val);
			res = TTMPSetFieldReal(msg, argv[i], (TReal) d);
		} else if (strcasecmp(type, "bool"  ) == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < INT32_MIN || l > INT32_MAX)
				usage("invalid Bool field value \"%s\"", val);
			res = TTMPSetFieldBool(msg, argv[i], (TBool) l);
		} else if (strcasecmp(type, "chr"   ) == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < 0 || l > UINT8_MAX)
				usage("invalid Chr field value \"%s\"", val);
			res = TTMPSetFieldChr(msg, argv[i], (TChr) l);
		} else if (strcasecmp(type, "unichr") == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1 || l < 0 || l > UINT16_MAX)
				usage("invalid UniChr field value \"%s\"", val);
			res = TTMPSetFieldUniChr(msg, argv[i], (TUniChr) l);
		} else if (strcasecmp(type, "long64") == 0) {
			if (sscanf(val, "%" SCANF_LL "d", &l) != 1)
				usage("invalid Long64 field value \"%s\"", val);
			res = TTMPSetFieldLong64(msg, argv[i], (TLong64) l);
		} else if (strcasecmp(type, "str"   ) == 0) {
			res = TTMPSetFieldStr(msg, argv[i], val);
		} else if (strcasecmp(type, "unistr") == 0) {
			usage("UniStr field not implemented yet");
		} else if (strcasecmp(type, "binary") == 0) {
			usage("Binary field not implemented yet");
		} else {
			usage("invalid field type \"%s\"", type);
		}
		if (res != TTMP_ERR_NONE)
			die("unable to set field \"%s\", error %d", argv[i], res);
	}

#ifdef DUMP_MESSAGE
	printf("Dump of message:\n");
	res = TTMPMessageDump(msg, stdout);
	if (res != TTMP_ERR_NONE)
		die("unable to dump message");
#endif

	printf("Connecting to \"%s\"...\n", addr);
	res = TTMPConnectionConnect(conn, addr);
	if (res != TTMP_ERR_NONE)
		die("unable to connect, error %d, errno %d", res, socket_errno);

	printf("Connected, sending message...\n");
	res = TTMPConnectionSend(conn, msg);
	if (res != TTMP_ERR_NONE)
		die("unable to send, error %d, errno %d", res, socket_errno);

	res = TTMPConnectionDestroy(conn);
	if (res != TTMP_ERR_NONE)
		die("unable to destroy connection, error %d", res);

	res = TTMPMessageDestroy(msg);
	if (res != TTMP_ERR_NONE)
		die("unable to destroy message, error %d", res);

	return 0;
}

/* EOF */
