/* TTMPSvr.c
 *
 * TTMPServer implementation.
 *
 * Copyright (C) 2005-2006 TomTom BV <http://www.tomtom.com/>
 * Author 0: Dimitry Andric <dimitry.andric@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2
 * as published by the Free Software Foundation.
 */

#include <stdlib.h>
#include <string.h>
#include "TTMP.h"
#include "TTMPSvr.h"
#include "TTMPConn.h"
#include "TTMPPriv.h"

#ifdef DMALLOC
#include <dmalloc.h>
#endif

int TTMPServerCreate(TTMPServer *svr, TInt32 timeout, TInt32 major, TInt32 minor)
{
	TTMPServer newsvr;

	if (svr == NULL)
		return TTMP_ERR_SVRNULL;
	CHECK_TIMEOUT(timeout);

	newsvr = (TTMPServer) malloc(sizeof(struct sTTMPServer));
	if (newsvr == NULL)
		return TTMP_ERR_NOMEMORY;

	newsvr->magic = TTMP_SVR_MAGIC;
	newsvr->timeout = (timeout == TTMP_TO_DEFAULT) ? TTMP_TO_DEFVAL : timeout;
	newsvr->major = major;
	newsvr->minor = minor;
	newsvr->address.domain = INVALID_DOMAIN;
	newsvr->socket = INVALID_SOCKET;

	/* Return server object to caller */
	*svr = newsvr;

	return TTMP_ERR_NONE;
}

int TTMPServerDestroy(TTMPServer svr)
{
	int res;

	CHECK_OBJ(SVR, svr);

	/* Close server if needed */
	res = TTMPServerClose(svr);

	/* Zap magic to provoke errors */
	svr->magic = 0;

	/* Delete server object */
	free(svr);

	return res;
}

int TTMPServerValidate(TTMPServer svr)
{
	CHECK_OBJ(SVR, svr);
	return TTMP_ERR_NONE;
}

int TTMPServerListen(TTMPServer svr, cTTMPAddress addr)
{
	int res;

	CHECK_OBJ(SVR, svr);

	/* Parse the address */
	res = TTMPSocketAddressParse(addr, &svr->address);
	if (res != TTMP_ERR_NONE)
		return res;

	/* Create socket if needed */
	if (SOCKET_INVALID(svr->socket)) {
		res = TTMPSocketCreate(&svr->socket, svr->address.domain);
		if (res != TTMP_ERR_NONE)
			return res;
	}

	/* Bind socket */
	res = TTMPSocketBind(svr->socket, &svr->address);
	if (res != TTMP_ERR_NONE)
		return res;

	/* Start listening */
	res = TTMPSocketListen(svr->socket, TTMP_SVR_MAXCONN);
	if (res != TTMP_ERR_NONE)
		return res;

	return TTMP_ERR_NONE;
}

int TTMPServerAccept(TTMPServer svr, TTMPConnection* newconn)
{
	TTMPSocket socket;
	int res;
	TTMPConnection conn;

	CHECK_OBJ(SVR, svr);
	if (newconn == NULL)
		return TTMP_ERR_ARGUMENT;

	if (SOCKET_INVALID(svr->socket))
		return TTMP_ERR_NOLISTEN;

	res = TTMPSocketAccept(svr->socket, &socket, svr->timeout);
	if (res != TTMP_ERR_NONE)
		return res;

	/* Create TTMPConnection object from socket */
	res = TTMPConnectionCreateInternal(&conn, svr->timeout, svr->major, svr->minor, socket);
	if (res != TTMP_ERR_NONE)
		return res;
	conn->sendstate = SS_CONNECTED;
	conn->recvstate = RS_CONNECTED;

	res = TTMPConnectionCheckVersion(conn, svr->timeout);
	if (res != TTMP_ERR_NONE) {
		/* Don't leak connections on error */
		TTMPConnectionDestroy(conn);
		return res;
	}

	/* Return connection object to caller */
	*newconn = conn;

	return TTMP_ERR_NONE;
}

int TTMPServerClose(TTMPServer svr)
{
	int res;

	CHECK_OBJ(SVR, svr);

	if (svr->address.domain != INVALID_DOMAIN) {
		/* Unbind socket, ignore errors */
		TTMPSocketUnbind(svr->socket, &svr->address);
		svr->address.domain = INVALID_DOMAIN;
	}

	if (SOCKET_VALID(svr->socket)) {
		res = TTMPSocketDestroy(svr->socket);
		svr->socket = INVALID_SOCKET;
	} else {
		res = TTMP_ERR_NONE;
	}

	return res;
}

/* EOF */
